#ifndef STOPWATCH_HH
#define STOPWATCH_HH

#include <ctime>

#include <stdexcept>  // provides std::runtime_error


/**
 * A class for time measurements.
 */
class Stopwatch
{
public:
  Stopwatch ()
    : running(false)
  {}

  // start time measurement
  void start ()
  {
    running = true;
    get(start_wall,start_usage);
  }

  // stop time measurement
  void stop ()
  {
    running = false;
    get(stop_wall,stop_usage);
  }

  // measured total time; also callable during time measurement
  double elapsed ()
  {
    if (running)
    {
      get(tmp_wall,tmp_usage);
      return measure(start_wall,tmp_wall);
    }
    return measure(start_wall,stop_wall);
  }

  // measured CPU time; also callable during time measurement
  double usage ()
  {
    if (running)
    {
      get(tmp_wall,tmp_usage);
      return measure(start_usage,tmp_usage);
    }
    return measure(start_usage,stop_usage);
  }

private:
  void get (timespec& wall, timespec& usage)
  {
    int r1, r2;
    r1 = clock_gettime(CLOCK_REALTIME, &wall);
    r2 = clock_gettime(CLOCK_THREAD_CPUTIME_ID, &usage);
    if (r1 != 0 || r2 != 0)
      throw std::runtime_error("Failed to initialize stopwatch");
  }

  double measure (timespec& start, timespec& stop)
  {
    double s  = stop.tv_sec - start.tv_sec;
    double ns = stop.tv_nsec - start.tv_nsec;
    return s + ns * 1e-9;
  }

private:
  timespec start_wall;
  timespec stop_wall;
  timespec start_usage;
  timespec stop_usage;
  timespec tmp_wall;
  timespec tmp_usage;
  bool running;
};


#endif // STOPWATCH_HH
