#include <cstddef>  // provides size_t
#include <cstdlib>  // provides atoi, srand, rand
#include <cstdio>   // provides printf

#include "test.hh"
#include "stopwatch.hh"


/**
 * Procedure for componentwise vector multiplication.
 */
void calc (size_t m,
           const Vec& A, const Vec& B, Vec& C)
{
  for (size_t i = 0; i < m; i++)
    calc_i(A[i],B[i],C[i]);
}

int main (int argc, char** argv)
{
  // to get a reliable measurement we want do the computation multiple times
  size_t computation_times = 10000000;
  if (argc > 1)
    computation_times = atoi(argv[1]);

  // create vectors
  Vec A(n);
  Vec B(n);
  Vec C(n);

  // initialize vectors
  srand(42);
  for (size_t i = 0; i < n; i++)
  {
    A[i][0] = rand();
    B[i][0] = rand();
    C[i][0] = 0.0;
  }

  // start time measurement
  Stopwatch watch;
  watch.start();

  // do computation multiple times
  for (size_t i = 0; i < computation_times; i++)
    calc(n,A,B,C);

  // stop time measurement and write measurement results
  watch.stop();
  printf("total time: %f, cpu time: %f\n",watch.elapsed(),watch.usage());

  // print computation result
  // for (size_t i = 0; i < n; i++)
  //   printf("%f - %f\n",C[i][0],A[i][0]*B[i][0]);

  // return without error
  return 0;
}
